@@ -153,16 +153,18 @@
     else
     {
       unsigned char*  p = bit->buffer;
-      int             i, j;
+      int             i;
 
 
-      for ( i = 0; i < bit->rows; i++ )
+      for ( i = 0; i < pitch; i += 3, p += 3 )
       {
-        for ( j = 0; j < bit->width; j++ )
-          memcpy( p + 3 * j, display->back_color.chroma, 3 );
-
-        p += pitch;
+        p[0] = display->back_color.chroma[0];
+        p[1] = display->back_color.chroma[1];
+        p[2] = display->back_color.chroma[2];
       }
+
+      for ( i = 1; i < bit->rows; i++, p += pitch )
+        memcpy( p, p - pitch, (size_t)pitch );
     }
   }
 
@@ -431,8 +433,8 @@
 
         if ( handle->preload )
         {
-          FILE*  file = fopen( filename, "rb" );
-          int    file_size;
+          FILE*   file = fopen( filename, "rb" );
+          size_t  file_size;
 
 
           if ( file == NULL )  /* shouldn't happen */
@@ -442,16 +444,33 @@
           }
 
           fseek( file, 0, SEEK_END );
-          file_size = ftell( file );
+          file_size = (size_t)ftell( file );
           fseek( file, 0, SEEK_SET );
 
           if ( file_size <= 0 )
+          {
+            free( font );
+            fclose( file );
             return FT_Err_Invalid_Stream_Operation;
+          }
 
-          font->file_address = malloc( (size_t)file_size );
-          fread( font->file_address, 1, (size_t)file_size, file );
+          font->file_address = malloc( file_size );
+          if ( !font->file_address )
+          {
+            free( font );
+            fclose( file );
+            return FT_Err_Out_Of_Memory;
+          }
 
-          font->file_size = (size_t)file_size;
+          if ( fread( font->file_address, 1, file_size, file ) != file_size )
+          {
+            free( font->file_address );
+            free( font );
+            fclose( file );
+            return FT_Err_Invalid_Stream_Read;
+          }
+
+          font->file_size = file_size;
 
           fclose( file );
         }
@@ -1088,9 +1107,9 @@
         glyph->vadvance.y = -metrics->vertAdvance;
 
         if ( prev_rsb_delta - face->glyph->lsb_delta >= 32 )
-          glyph->delta = -1 << 6;
+          glyph->delta = -1 * 64;
         else if ( prev_rsb_delta - face->glyph->lsb_delta < -32 )
-          glyph->delta = 1 << 6;
+          glyph->delta = 1 * 64;
         else
           glyph->delta = 0;
       }
@@ -1307,7 +1326,7 @@
         if ( sc->vertical )
         {
           bitmap->left += ( glyph->vvector.x + pen.x ) >> 6;
-          bitmap->top  += ( glyph->vvector.x + pen.y ) >> 6;
+          bitmap->top  += ( glyph->vvector.y + pen.y ) >> 6;
         }
         else
         {
